@@ -2,14 +2,12 @@ package JSON::Validator;
 use Mojo::Base -base;
 use Exporter 'import';
 
-use B;
 use Carp 'confess';
 use JSON::Validator::Formats;
 use JSON::Validator::Joi;
 use JSON::Validator::Ref;
 use JSON::Validator::Schema;
-use JSON::Validator::Util
-  qw(E data_checksum data_section data_type is_type json_pointer prefix_errors schema_type);
+use JSON::Validator::Util qw(E data_checksum data_section data_type is_type json_pointer prefix_errors schema_type);
 use List::Util 'uniq';
 use Mojo::File 'path';
 use Mojo::JSON qw(false true);
@@ -17,13 +15,11 @@ use Mojo::URL;
 use Mojo::Util qw(url_unescape sha1_sum);
 use Scalar::Util qw(blessed refaddr);
 
-use constant CASE_TOLERANT     => File::Spec->case_tolerant;
-use constant DEBUG             => $ENV{JSON_VALIDATOR_DEBUG} || 0;
-use constant RECURSION_LIMIT   => $ENV{JSON_VALIDATOR_RECURSION_LIMIT} || 100;
-use constant SPECIFICATION_URL => 'http://json-schema.org/draft-04/schema#';
-use constant YAML_SUPPORT      => eval 'use YAML::XS 0.67;1';
+use constant CASE_TOLERANT   => File::Spec->case_tolerant;
+use constant DEBUG           => $ENV{JSON_VALIDATOR_DEBUG} || 0;
+use constant RECURSION_LIMIT => $ENV{JSON_VALIDATOR_RECURSION_LIMIT} || 100;
 
-our $VERSION = '4.03';
+our $VERSION = '4.04';
 our @EXPORT_OK = qw(joi validate_json);
 
 our %SCHEMAS = (
@@ -36,8 +32,7 @@ my $BUNDLED_CACHE_DIR = path(path(__FILE__)->dirname, qw(Validator cache));
 my $HTTP_SCHEME_RE    = qr{^https?:};
 
 has cache_paths => sub {
-  return [split(/:/, $ENV{JSON_VALIDATOR_CACHE_PATH} || ''),
-    $BUNDLED_CACHE_DIR];
+  return [split(/:/, $ENV{JSON_VALIDATOR_CACHE_PATH} || ''), $BUNDLED_CACHE_DIR];
 };
 
 has formats => sub { shift->_build_formats };
@@ -66,7 +61,7 @@ sub bundle {
 
   my $schema    = $self->_new_schema($args->{schema} || $self->schema);
   my $schema_id = $schema->id || $self->{root_schema_url} || '';
-  my @topics = ([$schema->data, my $bundle = {}, '']);    # ([$from, $to], ...);
+  my @topics    = ([$schema->data, my $bundle = {}, '']);                 # ([$from, $to], ...);
 
   if ($args->{replace}) {
     $cloner = sub {
@@ -96,9 +91,7 @@ sub bundle {
 
       my $path = $self->_definitions_path($bundle, $tied);
       unless ($self->{bundled_refs}{$tied->fqn}++) {
-        push @topics,
-          [_node($schema->data, $path, 1, 0) || {},
-          _node($bundle, $path, 1, 1)];
+        push @topics, [_node($schema->data, $path, 1, 0) || {}, _node($bundle, $path, 1, 1)];
         push @topics, [$tied->schema, _node($bundle, $path, 0, 1)];
       }
 
@@ -136,11 +129,10 @@ sub coerce {
     $what = {booleans => 1, numbers => 1, strings => 1};
   }
 
-  state $short = {bool => 'booleans', def => 'defaults', num => 'numbers',
-    str => 'strings'};
+  state $short = {bool => 'booleans', def => 'defaults', num => 'numbers', str => 'strings'};
 
-  $what = {map { ($_ => 1) } split /,/, $what} unless ref $what;
-  $self->{coerce} = {};
+  $what                                 = {map { ($_ => 1) } split /,/, $what} unless ref $what;
+  $self->{coerce}                       = {};
   $self->{coerce}{($short->{$_} || $_)} = $what->{$_} for keys %$what;
 
   return $self;
@@ -149,8 +141,7 @@ sub coerce {
 sub get { JSON::Validator::Util::schema_extract(shift->schema->data, shift) }
 
 sub joi {
-  Mojo::Util::deprecated(
-    'JSON::Validator::joi() is replaced by JSON::Validator::Joi::joi().');
+  Mojo::Util::deprecated('JSON::Validator::joi() is replaced by JSON::Validator::Joi::joi().');
   return JSON::Validator::Joi->new unless @_;
   my ($data, $joi) = @_;
   return $joi->validate($data, $joi);
@@ -158,15 +149,14 @@ sub joi {
 
 sub load_and_validate_schema {
   my ($self, $schema, $args) = @_;
-  my $specification = $args->{schema} || SPECIFICATION_URL;
-  $self->{version} = $1
-    if !$self->{version} and $specification =~ /draft-0+(\w+)/;
 
-  my $obj = $self->_new_schema($schema, specification => $specification);
-  confess join "\n", "Invalid JSON specification $schema",
-    map {"- $_"} @{$obj->errors}
-    if @{$obj->errors};
-  $self->{schema} = $obj;
+  $self->{version} = $1 if !$self->{version} and ($args->{schema} || 'draft-04') =~ m!draft-0+(\w+)!;
+
+  my $schema_obj = $self->_new_schema($schema, %$args);
+  confess join "\n", "Invalid JSON specification $schema", map {"- $_"} @{$schema_obj->errors}
+    if @{$schema_obj->errors};
+
+  $self->{schema} = $schema_obj;
   return $self;
 }
 
@@ -193,9 +183,9 @@ sub validate {
   $schema //= $self->schema->data;
   return E '/', 'No validation rules defined.' unless defined $schema;
 
-  local $self->{schema} = $self->_new_schema($schema);
-  local $self->{seen}   = {};
-  local $self->{temp_schema} = [];    # make sure random-errors.t does not fail
+  local $self->{schema}      = $self->_new_schema($schema);
+  local $self->{seen}        = {};
+  local $self->{temp_schema} = [];                            # make sure random-errors.t does not fail
   my @errors = $self->_validate($_[1], '', $schema);
   return @errors;
 }
@@ -207,32 +197,31 @@ sub validate_json {
 
 sub _build_formats {
   return {
-    'byte'          => JSON::Validator::Formats->can('check_byte'),
-    'date'          => JSON::Validator::Formats->can('check_date'),
-    'date-time'     => JSON::Validator::Formats->can('check_date_time'),
-    'duration'      => JSON::Validator::Formats->can('check_duration'),
-    'double'        => JSON::Validator::Formats->can('check_double'),
-    'email'         => JSON::Validator::Formats->can('check_email'),
-    'float'         => JSON::Validator::Formats->can('check_float'),
-    'hostname'      => JSON::Validator::Formats->can('check_hostname'),
-    'idn-email'     => JSON::Validator::Formats->can('check_idn_email'),
-    'idn-hostname'  => JSON::Validator::Formats->can('check_idn_hostname'),
-    'int32'         => JSON::Validator::Formats->can('check_int32'),
-    'int64'         => JSON::Validator::Formats->can('check_int64'),
-    'ipv4'          => JSON::Validator::Formats->can('check_ipv4'),
-    'ipv6'          => JSON::Validator::Formats->can('check_ipv6'),
-    'iri'           => JSON::Validator::Formats->can('check_iri'),
-    'iri-reference' => JSON::Validator::Formats->can('check_iri_reference'),
-    'json-pointer'  => JSON::Validator::Formats->can('check_json_pointer'),
-    'regex'         => JSON::Validator::Formats->can('check_regex'),
-    'relative-json-pointer' =>
-      JSON::Validator::Formats->can('check_relative_json_pointer'),
-    'time'          => JSON::Validator::Formats->can('check_time'),
-    'uri'           => JSON::Validator::Formats->can('check_uri'),
-    'uri-reference' => JSON::Validator::Formats->can('check_uri_reference'),
-    'uri-reference' => JSON::Validator::Formats->can('check_uri_reference'),
-    'uri-template'  => JSON::Validator::Formats->can('check_uri_template'),
-    'uuid'          => JSON::Validator::Formats->can('check_uuid'),
+    'byte'                  => JSON::Validator::Formats->can('check_byte'),
+    'date'                  => JSON::Validator::Formats->can('check_date'),
+    'date-time'             => JSON::Validator::Formats->can('check_date_time'),
+    'duration'              => JSON::Validator::Formats->can('check_duration'),
+    'double'                => JSON::Validator::Formats->can('check_double'),
+    'email'                 => JSON::Validator::Formats->can('check_email'),
+    'float'                 => JSON::Validator::Formats->can('check_float'),
+    'hostname'              => JSON::Validator::Formats->can('check_hostname'),
+    'idn-email'             => JSON::Validator::Formats->can('check_idn_email'),
+    'idn-hostname'          => JSON::Validator::Formats->can('check_idn_hostname'),
+    'int32'                 => JSON::Validator::Formats->can('check_int32'),
+    'int64'                 => JSON::Validator::Formats->can('check_int64'),
+    'ipv4'                  => JSON::Validator::Formats->can('check_ipv4'),
+    'ipv6'                  => JSON::Validator::Formats->can('check_ipv6'),
+    'iri'                   => JSON::Validator::Formats->can('check_iri'),
+    'iri-reference'         => JSON::Validator::Formats->can('check_iri_reference'),
+    'json-pointer'          => JSON::Validator::Formats->can('check_json_pointer'),
+    'regex'                 => JSON::Validator::Formats->can('check_regex'),
+    'relative-json-pointer' => JSON::Validator::Formats->can('check_relative_json_pointer'),
+    'time'                  => JSON::Validator::Formats->can('check_time'),
+    'uri'                   => JSON::Validator::Formats->can('check_uri'),
+    'uri-reference'         => JSON::Validator::Formats->can('check_uri_reference'),
+    'uri-reference'         => JSON::Validator::Formats->can('check_uri_reference'),
+    'uri-template'          => JSON::Validator::Formats->can('check_uri_template'),
+    'uuid'                  => JSON::Validator::Formats->can('check_uuid'),
   };
 }
 
@@ -259,8 +248,8 @@ sub _definitions_path {
   my ($spec_path, $fragment) = split '#', $ref->fqn;
   my $key = $fragment;
   if (-e $spec_path) {
-    $key = join '-', map { s!^\W+!!; $_ } grep {$_} path($spec_path)->basename,
-      $fragment, substr(sha1_sum($spec_path), 0, 10);
+    $key = join '-', map { s!^\W+!!; $_ } grep {$_} path($spec_path)->basename, $fragment,
+      substr(sha1_sum($spec_path), 0, 10);
   }
 
   # Fallback or nicer path name
@@ -280,8 +269,7 @@ sub _load_schema {
 
   if ($url =~ m!^https?://!) {
     warn "[JSON::Validator] Loading schema from URL $url\n" if DEBUG;
-    return $self->_load_schema_from_url(Mojo::URL->new($url)->fragment(undef)),
-      "$url";
+    return $self->_load_schema_from_url(Mojo::URL->new($url)->fragment(undef)), "$url";
   }
 
   if ($url =~ m!^data://([^/]*)/(.*)!) {
@@ -302,13 +290,11 @@ sub _load_schema {
   if (-e $file) {
     $file = $file->realpath;
     warn "[JSON::Validator] Loading schema from file: $file\n" if DEBUG;
-    return $self->_load_schema_from_text(\$file->slurp),
-      CASE_TOLERANT ? path(lc $file) : $file;
+    return $self->_load_schema_from_text(\$file->slurp), CASE_TOLERANT ? path(lc $file) : $file;
   }
   elsif ($url =~ m!^/! and $self->ua->server->app) {
     warn "[JSON::Validator] Loading schema from URL $url\n" if DEBUG;
-    return $self->_load_schema_from_url(Mojo::URL->new($url)->fragment(undef)),
-      "$url";
+    return $self->_load_schema_from_url(Mojo::URL->new($url)->fragment(undef)), "$url";
   }
 
   confess "Unable to load schema '$url' ($file)";
@@ -316,17 +302,7 @@ sub _load_schema {
 
 sub _load_schema_from_text {
   my ($self, $text) = @_;
-
-  # JSON
-  return Mojo::JSON::decode_json($$text) if $$text =~ /^\s*\{/s;
-
-  # YAML
-  die "[JSON::Validator] YAML::XS 0.67 is missing or could not be loaded."
-    unless YAML_SUPPORT;
-
-  no warnings 'once';
-  local $YAML::XS::Boolean = 'JSON::PP';
-  return YAML::XS::Load($$text);
+  return $$text =~ /^\s*\{/s ? Mojo::JSON::decode_json($$text) : JSON::Validator::Util::_yaml_load($$text);
 }
 
 sub _load_schema_from_url {
@@ -346,14 +322,9 @@ sub _load_schema_from_url {
   confess "GET $url == $err"               if DEBUG and $err;
   die "[JSON::Validator] GET $url == $err" if $err;
 
-  if ($cache_path
-    and
-    ($cache_path ne $BUNDLED_CACHE_DIR or $ENV{JSON_VALIDATOR_CACHE_ANYWAYS})
-    and -w $cache_path)
-  {
+  if ($cache_path and ($cache_path ne $BUNDLED_CACHE_DIR or $ENV{JSON_VALIDATOR_CACHE_ANYWAYS}) and -w $cache_path) {
     $cache_file = path $cache_path, $cache_file;
-    warn "[JSON::Validator] Caching $url to $cache_file\n"
-      unless $ENV{HARNESS_ACTIVE};
+    warn "[JSON::Validator] Caching $url to $cache_file\n" unless $ENV{HARNESS_ACTIVE};
     $cache_file->spurt($tx->res->body);
   }
 
@@ -361,23 +332,25 @@ sub _load_schema_from_url {
 }
 
 sub _new_schema {
-  my ($self, $spec, @attrs) = @_;
-  return $spec if blessed $spec and $spec->can('specification');
+  my ($self, $schema, %attrs) = @_;
+  return $schema if blessed $schema and $schema->can('specification');
 
-  if (!$spec and is_type $spec, 'HASH') {
-    $spec = $spec->{'$spec'} || $spec->{schema};
+  # Compat with load_and_validate_schema()
+  $attrs{specification} = delete $attrs{schema} if $attrs{schema};
+
+  if (!$attrs{specification} and is_type $schema, 'HASH') {
+    $attrs{specification} = $schema->{'$schema'} if $schema->{'$schema'};
   }
-  if (!$spec and $self->{version}) {
-    $spec = sprintf 'http://json-schema.org/draft-%02s/schema#',
-      $self->{version};
+  if (!$attrs{specification} and $self->{version}) {
+    $attrs{specification} = sprintf 'http://json-schema.org/draft-%02s/schema#', $self->{version};
   }
 
-  push @attrs, formats => $self->{formats} if $self->{formats};
-  push @attrs, version => $self->{version} if $self->{version};
-  unshift @attrs, $spec, map { ($_ => $self->$_) } qw(cache_paths formats ua);
-  my $schema = $self->_schema_class($spec)->new(@attrs);
-  $schema->specification($spec) if $spec and !$schema->specification;
-  return $schema;
+  $attrs{formats} ||= $self->{formats} if $self->{formats};
+  $attrs{version} ||= $self->{version} if $self->{version};
+  $attrs{$_} = $self->$_ for qw(cache_paths ua);
+
+  my $schema_obj = $self->_schema_class($attrs{specification} || $schema)->new($schema, %attrs);
+  return $schema_obj;
 }
 
 sub _node {
@@ -399,8 +372,7 @@ sub _ref_to_schema {
   my @guard;
   while (my $tied = tied %$schema) {
     push @guard, $tied->ref;
-    confess "Seems like you have a circular reference: @guard"
-      if @guard > RECURSION_LIMIT;
+    confess "Seems like you have a circular reference: @guard" if @guard > RECURSION_LIMIT;
     $schema = $tied->schema;
     last if is_type $schema, 'BOOL';
   }
@@ -443,15 +415,11 @@ sub _resolve {
   unless ($self->{level}) {
     my $rid = $schema->{$id_key} // $id;
     if ($rid) {
-      confess "Root schema cannot have a fragment in the 'id'. ($rid)"
-        if $rid =~ /\#./;
-      confess "Root schema cannot have a relative 'id'. ($rid)"
-        unless $rid =~ /^\w+:/
-        or -e $rid
-        or $rid =~ m!^/!;
+      confess "Root schema cannot have a fragment in the 'id'. ($rid)" if $rid =~ /\#./;
+      confess "Root schema cannot have a relative 'id'. ($rid)" unless $rid =~ /^\w+:/ or -e $rid or $rid =~ m!^/!;
     }
     warn sprintf "[JSON::Validator] Using root_schema_url of '$rid'\n" if DEBUG;
-    $self->id($rid) if $self->can('id') and !$self->id;
+    $self->id($rid)                                                    if $self->can('id') and !$self->id;
     $self->{root_schema_url} = $rid;
   }
 
@@ -459,8 +427,7 @@ sub _resolve {
   $self->_register_schema($schema, $id);
 
   my (%seen, @refs);
-  my @topics
-    = ([$schema, is_type($id, 'Mojo::File') ? $id : Mojo::URL->new($id)]);
+  my @topics = ([$schema, is_type($id, 'Mojo::File') ? $id : Mojo::URL->new($id)]);
   while (@topics) {
     my ($topic, $base) = @{shift @topics};
 
@@ -471,8 +438,7 @@ sub _resolve {
       my $seen_addr = join ':', $base, refaddr($topic);
       next if $seen{$seen_addr}++;
 
-      push @refs, [$topic, $base] and next
-        if $topic->{'$ref'} and !ref $topic->{'$ref'};
+      push @refs, [$topic, $base] and next if $topic->{'$ref'} and !ref $topic->{'$ref'};
 
       if ($topic->{$id_key} and !ref $topic->{$id_key}) {
         my $fqn = Mojo::URL->new($topic->{$id_key});
@@ -514,9 +480,8 @@ sub _resolve_ref {
   while (1) {
     last if is_type $other, 'BOOL';
     push @guard, ($ref = $other->{'$ref'});
-    confess "Seems like you have a circular reference: @guard"
-      if @guard > RECURSION_LIMIT;
-    last if !$ref or ref $ref;
+    confess "Seems like you have a circular reference: @guard" if @guard > RECURSION_LIMIT;
+    last                                                       if !$ref or ref $ref;
     $fqn = $ref =~ m!^/! ? "#$ref" : $ref;
     my ($location, $pointer) = split /#/, $fqn, 2;
     $url     = $location = _location_to_abs($location, $url);
@@ -527,9 +492,7 @@ sub _resolve_ref {
 
     if (defined $pointer and length $pointer and $pointer =~ m!^/!) {
       $other = Mojo::JSON::Pointer->new($other)->get($pointer);
-      confess
-        qq[Possibly a typo in schema? Could not find "$pointer" in "$location" ($ref)]
-        if not defined $other;
+      confess qq[Possibly a typo in schema? Could not find "$pointer" in "$location" ($ref)] if not defined $other;
     }
   }
 
@@ -542,39 +505,32 @@ sub _schema_class {
 
   my $schema_class = $spec && $SCHEMAS{$spec} || 'JSON::Validator::Schema';
   $schema_class =~ s!^\+(.+)$!JSON::Validator::Schema::$1!;
-  confess "Could not load $schema_class: $@"
-    unless $schema_class->can('new')
-    or eval "require $schema_class;1";
+  confess "Could not load $schema_class: $@" unless $schema_class->can('new') or eval "require $schema_class;1";
 
   return $schema_class if ref $_[0] eq __PACKAGE__;
 
-  my $jv_class = ref($self) || $self;
-  my $short_schema_class
-    = $schema_class =~ m!JSON::Validator::Schema::(.+)! ? $1 : $schema_class;
-  my $package = sprintf 'JSON::Validator::Schema::Backcompat::%s',
+  my $jv_class           = ref($self) || $self;
+  my $short_schema_class = $schema_class =~ m!JSON::Validator::Schema::(.+)! ? $1 : $schema_class;
+  my $package            = sprintf 'JSON::Validator::Schema::Backcompat::%s',
     $jv_class =~ m!^JSON::Validator::(.+)! ? $1 : $jv_class;
   return $package if $package->can('new');
 
-  die "package $package: $@"
-    unless eval "package $package; use Mojo::Base '$jv_class'; 1";
+  die "package $package: $@" unless eval "package $package; use Mojo::Base '$jv_class'; 1";
   Mojo::Util::monkey_patch($package, $_ => $schema_class->can($_))
-    for
-    qw(bundle contains data errors get id new resolve specification validate);
+    for qw(bundle contains data errors get id new resolve specification validate);
   return $package;
 }
 
 sub _validate {
   my ($self, $data, $path, $schema) = @_;
 
-  $schema = $self->_ref_to_schema($schema)
-    if ref $schema eq 'HASH' and $schema->{'$ref'};
+  $schema = $self->_ref_to_schema($schema) if ref $schema eq 'HASH' and $schema->{'$ref'};
   return $schema ? () : E $path, [not => 'not'] if is_type $schema, 'BOOL';
 
   my @errors;
 
   if ($self->recursive_data_protection) {
-    my $seen_addr = join ':', refaddr($schema),
-      (ref $data ? refaddr $data : ++$self->{seen}{scalar});
+    my $seen_addr = join ':', refaddr($schema), (ref $data ? refaddr $data : ++$self->{seen}{scalar});
 
     # Avoid recursion
     return @{$self->{seen}{$seen_addr}} if $self->{seen}{$seen_addr};
@@ -582,16 +538,14 @@ sub _validate {
     $self->{seen}{$seen_addr} = \@errors;
   }
 
-  my $to_json
-    = (blessed $data and $data->can('TO_JSON')) ? \$data->TO_JSON : undef;
+  my $to_json = (blessed $data and $data->can('TO_JSON')) ? \$data->TO_JSON : undef;
   $data = $$to_json if $to_json;
   my $type = $schema->{type} || schema_type $schema, $data;
 
   # Test base schema before allOf, anyOf or oneOf
   if (ref $type eq 'ARRAY') {
     push @{$self->{temp_schema}}, [map { +{%$schema, type => $_} } @$type];
-    return $self->_validate_any_of_types($to_json ? $$to_json : $_[1],
-      $path, $self->{temp_schema}[-1]);
+    return $self->_validate_any_of_types($to_json ? $$to_json : $_[1], $path, $self->{temp_schema}[-1]);
   }
   elsif ($type) {
     my $method = sprintf '_validate_type_%s', $type;
@@ -600,14 +554,12 @@ sub _validate {
   }
 
   if (exists $schema->{const}) {
-    push @errors,
-      $self->_validate_type_const($to_json ? $$to_json : $_[1], $path, $schema);
+    push @errors, $self->_validate_type_const($to_json ? $$to_json : $_[1], $path, $schema);
     return @errors if @errors;
   }
 
   if ($schema->{enum}) {
-    push @errors,
-      $self->_validate_type_enum($to_json ? $$to_json : $_[1], $path, $schema);
+    push @errors, $self->_validate_type_enum($to_json ? $$to_json : $_[1], $path, $schema);
     return @errors if @errors;
   }
 
@@ -617,25 +569,19 @@ sub _validate {
   }
 
   if (my $rules = $schema->{allOf}) {
-    push @errors,
-      $self->_validate_all_of($to_json ? $$to_json : $_[1], $path, $rules);
+    push @errors, $self->_validate_all_of($to_json ? $$to_json : $_[1], $path, $rules);
   }
 
   if (my $rules = $schema->{anyOf}) {
-    push @errors,
-      $self->_validate_any_of($to_json ? $$to_json : $_[1], $path, $rules);
+    push @errors, $self->_validate_any_of($to_json ? $$to_json : $_[1], $path, $rules);
   }
 
   if (my $rules = $schema->{oneOf}) {
-    push @errors,
-      $self->_validate_one_of($to_json ? $$to_json : $_[1], $path, $rules);
+    push @errors, $self->_validate_one_of($to_json ? $$to_json : $_[1], $path, $rules);
   }
 
   if ($schema->{if}) {
-    my $rules
-      = $self->_validate($data, $path, $schema->{if})
-      ? $schema->{else}
-      : $schema->{then};
+    my $rules = $self->_validate($data, $path, $schema->{if}) ? $schema->{else} : $schema->{then};
     push @errors, $self->_validate($data, $path, $rules // {});
   }
 
@@ -660,8 +606,7 @@ sub _validate_all_of {
 
   return prefix_errors(allOf => @errors_with_prefix)
     if @errors == 1
-    or
-    (grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors);
+    or (grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors);
 
   # combine all 'type' errors at the base path together
   my @details    = map $_->details, @errors;
@@ -679,9 +624,7 @@ sub _validate_any_of_types {
   }
 
   # favour a non-type error from one of the rules
-  if (my @e
-    = grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors)
-  {
+  if (my @e = grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors) {
     return @e;
   }
 
@@ -707,8 +650,7 @@ sub _validate_any_of {
 
   return prefix_errors(anyOf => @errors_with_prefix)
     if @errors == 1
-    or
-    (grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors);
+    or (grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors);
 
   # combine all 'type' errors at the base path together
   my @details    = map $_->details, @errors;
@@ -736,8 +678,7 @@ sub _validate_one_of {
 
   return prefix_errors(oneOf => @errors_with_prefix)
     if @errors == 1
-    or
-    (grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors);
+    or (grep { $_->details->[1] ne 'type' or $_->path ne ($path || '/') } @errors);
 
   # the type didn't match any of the rules: combine the errors together
   my @details    = map $_->details, @errors;
@@ -751,19 +692,15 @@ sub _validate_number_max {
 
   my $cmp_with = $schema->{exclusiveMaximum} // '';
   if (is_type $cmp_with, 'BOOL') {
-    push @errors, E $path,
-      [$expected => ex_maximum => $value, $schema->{maximum}]
-      unless $value < $schema->{maximum};
+    push @errors, E $path, [$expected => ex_maximum => $value, $schema->{maximum}] unless $value < $schema->{maximum};
   }
   elsif (is_type $cmp_with, 'NUM') {
-    push @errors, E $path, [$expected => ex_maximum => $value, $cmp_with]
-      unless $value < $cmp_with;
+    push @errors, E $path, [$expected => ex_maximum => $value, $cmp_with] unless $value < $cmp_with;
   }
 
   if (exists $schema->{maximum}) {
     my $cmp_with = $schema->{maximum};
-    push @errors, E $path, [$expected => maximum => $value, $cmp_with]
-      unless $value <= $cmp_with;
+    push @errors, E $path, [$expected => maximum => $value, $cmp_with] unless $value <= $cmp_with;
   }
 
   return @errors;
@@ -775,19 +712,15 @@ sub _validate_number_min {
 
   my $cmp_with = $schema->{exclusiveMinimum} // '';
   if (is_type $cmp_with, 'BOOL') {
-    push @errors, E $path,
-      [$expected => ex_minimum => $value, $schema->{minimum}]
-      unless $value > $schema->{minimum};
+    push @errors, E $path, [$expected => ex_minimum => $value, $schema->{minimum}] unless $value > $schema->{minimum};
   }
   elsif (is_type $cmp_with, 'NUM') {
-    push @errors, E $path, [$expected => ex_minimum => $value, $cmp_with]
-      unless $value > $cmp_with;
+    push @errors, E $path, [$expected => ex_minimum => $value, $cmp_with] unless $value > $cmp_with;
   }
 
   if (exists $schema->{minimum}) {
     my $cmp_with = $schema->{minimum};
-    push @errors, E $path, [$expected => minimum => $value, $cmp_with]
-      unless $value >= $cmp_with;
+    push @errors, E $path, [$expected => minimum => $value, $cmp_with] unless $value >= $cmp_with;
   }
 
   return @errors;
@@ -802,8 +735,7 @@ sub _validate_type_enum {
     return if $m eq data_checksum $i;
   }
 
-  $enum = join ', ',
-    map { (!defined or ref) ? Mojo::JSON::encode_json($_) : $_ } @$enum;
+  $enum = join ', ', map { (!defined or ref) ? Mojo::JSON::encode_json($_) : $_ } @$enum;
   return E $path, [enum => enum => $enum];
 }
 
@@ -818,8 +750,7 @@ sub _validate_type_const {
 sub _validate_format {
   my ($self, $value, $path, $schema) = @_;
   my $code = $self->formats->{$schema->{format}};
-  return do { warn "Format rule for '$schema->{format}' is missing"; return }
-    unless $code;
+  return do { warn "Format rule for '$schema->{format}' is missing"; return } unless $code;
   return unless my $err = $code->($value);
   return E $path, [format => $schema->{format}, $err];
 }
@@ -834,12 +765,10 @@ sub _validate_type_array {
     return E $path, [array => type => data_type $data];
   }
   if (defined $schema->{minItems} and $schema->{minItems} > @$data) {
-    push @errors, E $path,
-      [array => minItems => int(@$data), $schema->{minItems}];
+    push @errors, E $path, [array => minItems => int(@$data), $schema->{minItems}];
   }
   if (defined $schema->{maxItems} and $schema->{maxItems} < @$data) {
-    push @errors, E $path,
-      [array => maxItems => int(@$data), $schema->{maxItems}];
+    push @errors, E $path, [array => maxItems => int(@$data), $schema->{maxItems}];
   }
   if ($schema->{uniqueItems}) {
     my %uniq;
@@ -874,8 +803,7 @@ sub _validate_type_array {
       }
     }
     elsif (!$additional_items) {
-      push @errors, E $path,
-        [array => additionalItems => int(@$data), int(@rules)];
+      push @errors, E $path, [array => additionalItems => int(@$data), int(@rules)];
     }
   }
   elsif (exists $schema->{items}) {
@@ -927,19 +855,16 @@ sub _validate_type_number {
   }
   unless (is_type $value, 'NUM') {
     return E $path, [$expected => type => data_type $value]
-      if !$self->{coerce}{numbers}
-      or $value !~ /^-?(?:0|[1-9]\d*)(?:\.\d+)?(?:[eE][+-]?\d+)?$/;
+      if !$self->{coerce}{numbers} or $value !~ /^-?(?:0|[1-9]\d*)(?:\.\d+)?(?:[eE][+-]?\d+)?$/;
     $_[1] = 0 + $value;    # coerce input value
   }
 
-  push @errors, $self->_validate_format($value, $path, $schema)
-    if $schema->{format};
+  push @errors, $self->_validate_format($value, $path, $schema) if $schema->{format};
   push @errors, $self->_validate_number_max($value, $path, $schema, $expected);
   push @errors, $self->_validate_number_min($value, $path, $schema, $expected);
 
   my $d = $schema->{multipleOf};
-  push @errors, E $path, [$expected => multipleOf => $d]
-    if $d and ($value / $d) =~ /\.[^0]+$/;
+  push @errors, E $path, [$expected => multipleOf => $d] if $d and ($value / $d) =~ /\.[^0]+$/;
 
   return @errors;
 }
@@ -947,18 +872,15 @@ sub _validate_type_number {
 sub _validate_type_object {
   my ($self, $data, $path, $schema) = @_;
 
-  return E $path, [object => type => data_type $data]
-    unless ref $data eq 'HASH';
+  return E $path, [object => type => data_type $data] unless ref $data eq 'HASH';
 
   my @errors;
   my @dkeys = sort keys %$data;
   if (defined $schema->{maxProperties} and $schema->{maxProperties} < @dkeys) {
-    push @errors, E $path,
-      [object => maxProperties => int(@dkeys), $schema->{maxProperties}];
+    push @errors, E $path, [object => maxProperties => int(@dkeys), $schema->{maxProperties}];
   }
   if (defined $schema->{minProperties} and $schema->{minProperties} > @dkeys) {
-    push @errors, E $path,
-      [object => minProperties => int(@dkeys), $schema->{minProperties}];
+    push @errors, E $path, [object => minProperties => int(@dkeys), $schema->{minProperties}];
   }
   if (my $n_schema = $schema->{propertyNames}) {
     for my $name (keys %$data) {
@@ -971,11 +893,7 @@ sub _validate_type_object {
   for my $k (keys %{$schema->{properties}}) {
     my $r = $schema->{properties}{$k};
     push @{$rules{$k}}, $r;
-    if (  $self->{coerce}{defaults}
-      and ref $r eq 'HASH'
-      and exists $r->{default}
-      and !exists $data->{$k})
-    {
+    if ($self->{coerce}{defaults} and ref $r eq 'HASH' and exists $r->{default} and !exists $data->{$k}) {
       $data->{$k} = $r->{default};
     }
   }
@@ -985,10 +903,7 @@ sub _validate_type_object {
     push @{$rules{$_}}, $r for sort grep { $_ =~ /$p/ } @dkeys;
   }
 
-  my $additional
-    = exists $schema->{additionalProperties}
-    ? $schema->{additionalProperties}
-    : {};
+  my $additional = exists $schema->{additionalProperties} ? $schema->{additionalProperties} : {};
   if ($additional) {
     $additional = {} unless is_type $additional, 'HASH';
     $rules{$_} ||= [$additional] for @dkeys;
@@ -1013,8 +928,7 @@ sub _validate_type_object {
         grep { !exists $data->{$_} } @{$dependencies->{$k}};
     }
     elsif (ref $dependencies->{$k} eq 'HASH') {
-      push @errors,
-        $self->_validate_type_object($data, $path, $schema->{dependencies}{$k});
+      push @errors, $self->_validate_type_object($data, $path, $schema->{dependencies}{$k});
     }
   }
 
@@ -1025,12 +939,8 @@ sub _validate_type_object {
       my @e = $self->_validate($data->{$k}, json_pointer($path, $k), $r);
       push @errors, @e;
       next if @e or !is_type $r, 'HASH';
-      push @errors,
-        $self->_validate_type_enum($data->{$k}, json_pointer($path, $k), $r)
-        if $r->{enum};
-      push @errors,
-        $self->_validate_type_const($data->{$k}, json_pointer($path, $k), $r)
-        if $r->{const};
+      push @errors, $self->_validate_type_enum($data->{$k}, json_pointer($path, $k), $r)  if $r->{enum};
+      push @errors, $self->_validate_type_const($data->{$k}, json_pointer($path, $k), $r) if $r->{const};
     }
   }
 
@@ -1044,12 +954,8 @@ sub _validate_type_string {
   if (!defined $value or ref $value) {
     return E $path, [string => type => data_type $value];
   }
-  if (  B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK)
-    and 0 + $value eq $value
-    and $value * 0 == 0)
-  {
-    return E $path, [string => type => data_type $value]
-      unless $self->{coerce}{strings};
+  if (B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK) and 0 + $value eq $value and $value * 0 == 0) {
+    return E $path, [string => type => data_type $value] unless $self->{coerce}{strings};
     $_[1] = "$value";    # coerce input value
   }
   if ($schema->{format}) {
@@ -1057,14 +963,12 @@ sub _validate_type_string {
   }
   if (defined $schema->{maxLength}) {
     if (length($value) > $schema->{maxLength}) {
-      push @errors, E $path,
-        [string => maxLength => length($value), $schema->{maxLength}];
+      push @errors, E $path, [string => maxLength => length($value), $schema->{maxLength}];
     }
   }
   if (defined $schema->{minLength}) {
     if (length($value) < $schema->{minLength}) {
-      push @errors, E $path,
-        [string => minLength => length($value), $schema->{minLength}];
+      push @errors, E $path, [string => minLength => length($value), $schema->{minLength}];
     }
   }
   if (defined $schema->{pattern}) {
@@ -1136,8 +1040,7 @@ L<JSON::Validator::Joi> to define the schema programmatically.
 
 L<JSON::Validator> can load JSON schemas in multiple formats: Plain perl data
 structured (as shown in L</SYNOPSIS>), JSON or YAML. The JSON parsing is done
-with L<Mojo::JSON>, while YAML files require the optional module L<YAML::XS> to
-be installed.
+with L<Mojo::JSON>, while YAML files requires L<YAML::PP> or L<YAML::XS>.
 
 =head2 Resources
 
